from collections import defaultdict, deque


def bfs(start, max_distance, graph):
    visited = set()
    queue = deque([(start, 0)])
    while queue:
        node, distance = queue.popleft()
        if distance < max_distance:
            if node not in visited:
                visited.add(node)
                for neighbor in graph[node]:
                    queue.append((neighbor, distance + 1))
    return visited


def find_guarded_storages(n, m, roads, guards):
    graph = defaultdict(list)

    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)

    guarded_storages = set()

    for p, h in guards:
        reachable_storages = bfs(p, h, graph)
        guarded_storages.update(reachable_storages)

    return sorted(list(guarded_storages))


if __name__ == "__main__":
    import sys

    input = sys.stdin.read
    data = input().splitlines()

    n, m, k = map(int, data[0].split())

    roads = []
    for i in range(1, m + 1):
        a, b = map(int, data[i].split())
        roads.append((a, b))

    guards = []
    for i in range(m + 1, m + 1 + k):
        p, h = map(int, data[i].split())
        guards.append((p, h))

    result = find_guarded_storages(n, m, roads, guards)

    print(len(result))
    if result:
        print(' '.join(map(str, result)))